export const root: string
export const menu_icon: string
export const context_menu: string
export const horizontal: string
export const vertical: string
export const divider: string
export const active: string
export const menu: string
export const above: string
export const below: string
export const caret: string
export const down: string
export const up: string
export const left: string
export const right: string
export default ``
